+{
   locale_version => 1.27,
   entry => <<'ENTRY', # for DUCET v10.0.0
010D      ; [.1CE1.0020.0002] # LATIN SMALL LETTER C WITH CARON
0063 030C ; [.1CE1.0020.0002] # LATIN SMALL LETTER C WITH CARON
010C      ; [.1CE1.0020.0008] # LATIN CAPITAL LETTER C WITH CARON
0043 030C ; [.1CE1.0020.0008] # LATIN CAPITAL LETTER C WITH CARON
0161      ; [.1ED8.0020.0002] # LATIN SMALL LETTER S WITH CARON
0073 030C ; [.1ED8.0020.0002] # LATIN SMALL LETTER S WITH CARON
0160      ; [.1ED8.0020.0008] # LATIN CAPITAL LETTER S WITH CARON
0053 030C ; [.1ED8.0020.0008] # LATIN CAPITAL LETTER S WITH CARON
017E      ; [.1F88.0020.0002] # LATIN SMALL LETTER Z WITH CARON
007A 030C ; [.1F88.0020.0002] # LATIN SMALL LETTER Z WITH CARON
017D      ; [.1F88.0020.0008] # LATIN CAPITAL LETTER Z WITH CARON
005A 030C ; [.1F88.0020.0008] # LATIN CAPITAL LETTER Z WITH CARON
ENTRY
};
