# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 13.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V30
70400
70404
70405
70413
70415
70417
70419
70441
70442
70449
70450
70452
70453
70458
70460
70469
70471
70473
70475
70478
70480
70481
70487
70488
70493
70500
70502
70509
70512
70517
END
