# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 13.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.



# The name this table is to be known by, with the format of the mappings in
# the main body of the table, and what all code points missing from this file
# map to.
$Unicode::UCD::SwashInfo{'ToSB'}{'format'} = 's'; # string
$Unicode::UCD::SwashInfo{'ToSB'}{'missing'} = 'Other';

return <<'END';
9		Sp
A		LF
B	C	Sp
D		CR
20		Sp
21		STerm
22		Close
27	29	Close
2C	2D	SContinue
2E		ATerm
30	39	Numeric
3A		SContinue
3F		STerm
41	5A	Upper
5B		Close
5D		Close
61	7A	Lower
7B		Close
7D		Close
85		Sep
A0		Sp
AA		Lower
AB		Close
AD		Format
B5		Lower
BA		Lower
BB		Close
C0	D6	Upper
D8	DE	Upper
DF	F6	Lower
F8	FF	Lower
100		Upper
101		Lower
102		Upper
103		Lower
104		Upper
105		Lower
106		Upper
107		Lower
108		Upper
109		Lower
10A		Upper
10B		Lower
10C		Upper
10D		Lower
10E		Upper
10F		Lower
110		Upper
111		Lower
112		Upper
113		Lower
114		Upper
115		Lower
116		Upper
117		Lower
118		Upper
119		Lower
11A		Upper
11B		Lower
11C		Upper
11D		Lower
11E		Upper
11F		Lower
120		Upper
121		Lower
122		Upper
123		Lower
124		Upper
125		Lower
126		Upper
127		Lower
128		Upper
129		Lower
12A		Upper
12B		Lower
12C		Upper
12D		Lower
12E		Upper
12F		Lower
130		Upper
131		Lower
132		Upper
133		Lower
134		Upper
135		Lower
136		Upper
137	138	Lower
139		Upper
13A		Lower
13B		Upper
13C		Lower
13D		Upper
13E		Lower
13F		Upper
140		Lower
141		Upper
142		Lower
143		Upper
144		Lower
145		Upper
146		Lower
147		Upper
148	149	Lower
14A		Upper
14B		Lower
14C		Upper
14D		Lower
14E		Upper
14F		Lower
150		Upper
151		Lower
152		Upper
153		Lower
154		Upper
155		Lower
156		Upper
157		Lower
158		Upper
159		Lower
15A		Upper
15B		Lower
15C		Upper
15D		Lower
15E		Upper
15F		Lower
160		Upper
161		Lower
162		Upper
163		Lower
164		Upper
165		Lower
166		Upper
167		Lower
168		Upper
169		Lower
16A		Upper
16B		Lower
16C		Upper
16D		Lower
16E		Upper
16F		Lower
170		Upper
171		Lower
172		Upper
173		Lower
174		Upper
175		Lower
176		Upper
177		Lower
178	179	Upper
17A		Lower
17B		Upper
17C		Lower
17D		Upper
17E	180	Lower
181	182	Upper
183		Lower
184		Upper
185		Lower
186	187	Upper
188		Lower
189	18B	Upper
18C	18D	Lower
18E	191	Upper
192		Lower
193	194	Upper
195		Lower
196	198	Upper
199	19B	Lower
19C	19D	Upper
19E		Lower
19F	1A0	Upper
1A1		Lower
1A2		Upper
1A3		Lower
1A4		Upper
1A5		Lower
1A6	1A7	Upper
1A8		Lower
1A9		Upper
1AA	1AB	Lower
1AC		Upper
1AD		Lower
1AE	1AF	Upper
1B0		Lower
1B1	1B3	Upper
1B4		Lower
1B5		Upper
1B6		Lower
1B7	1B8	Upper
1B9	1BA	Lower
1BB		OLetter
1BC		Upper
1BD	1BF	Lower
1C0	1C3	OLetter
1C4	1C5	Upper
1C6		Lower
1C7	1C8	Upper
1C9		Lower
1CA	1CB	Upper
1CC		Lower
1CD		Upper
1CE		Lower
1CF		Upper
1D0		Lower
1D1		Upper
1D2		Lower
1D3		Upper
1D4		Lower
1D5		Upper
1D6		Lower
1D7		Upper
1D8		Lower
1D9		Upper
1DA		Lower
1DB		Upper
1DC	1DD	Lower
1DE		Upper
1DF		Lower
1E0		Upper
1E1		Lower
1E2		Upper
1E3		Lower
1E4		Upper
1E5		Lower
1E6		Upper
1E7		Lower
1E8		Upper
1E9		Lower
1EA		Upper
1EB		Lower
1EC		Upper
1ED		Lower
1EE		Upper
1EF	1F0	Lower
1F1	1F2	Upper
1F3		Lower
1F4		Upper
1F5		Lower
1F6	1F8	Upper
1F9		Lower
1FA		Upper
1FB		Lower
1FC		Upper
1FD		Lower
1FE		Upper
1FF		Lower
200		Upper
201		Lower
202		Upper
203		Lower
204		Upper
205		Lower
206		Upper
207		Lower
208		Upper
209		Lower
20A		Upper
20B		Lower
20C		Upper
20D		Lower
20E		Upper
20F		Lower
210		Upper
211		Lower
212		Upper
213		Lower
214		Upper
215		Lower
216		Upper
217		Lower
218		Upper
219		Lower
21A		Upper
21B		Lower
21C		Upper
21D		Lower
21E		Upper
21F		Lower
220		Upper
221		Lower
222		Upper
223		Lower
224		Upper
225		Lower
226		Upper
227		Lower
228		Upper
229		Lower
22A		Upper
22B		Lower
22C		Upper
22D		Lower
22E		Upper
22F		Lower
230		Upper
231		Lower
232		Upper
233	239	Lower
23A	23B	Upper
23C		Lower
23D	23E	Upper
23F	240	Lower
241		Upper
242		Lower
243	246	Upper
247		Lower
248		Upper
249		Lower
24A		Upper
24B		Lower
24C		Upper
24D		Lower
24E		Upper
24F	293	Lower
294		OLetter
295	2B8	Lower
2B9	2BF	OLetter
2C0	2C1	Lower
2C6	2D1	OLetter
2E0	2E4	Lower
2EC		OLetter
2EE		OLetter
300	36F	Extend
370		Upper
371		Lower
372		Upper
373		Lower
374		OLetter
376		Upper
377		Lower
37A	37D	Lower
37F		Upper
386		Upper
388	38A	Upper
38C		Upper
38E	38F	Upper
390		Lower
391	3A1	Upper
3A3	3AB	Upper
3AC	3CE	Lower
3CF		Upper
3D0	3D1	Lower
3D2	3D4	Upper
3D5	3D7	Lower
3D8		Upper
3D9		Lower
3DA		Upper
3DB		Lower
3DC		Upper
3DD		Lower
3DE		Upper
3DF		Lower
3E0		Upper
3E1		Lower
3E2		Upper
3E3		Lower
3E4		Upper
3E5		Lower
3E6		Upper
3E7		Lower
3E8		Upper
3E9		Lower
3EA		Upper
3EB		Lower
3EC		Upper
3ED		Lower
3EE		Upper
3EF	3F3	Lower
3F4		Upper
3F5		Lower
3F7		Upper
3F8		Lower
3F9	3FA	Upper
3FB	3FC	Lower
3FD	42F	Upper
430	45F	Lower
460		Upper
461		Lower
462		Upper
463		Lower
464		Upper
465		Lower
466		Upper
467		Lower
468		Upper
469		Lower
46A		Upper
46B		Lower
46C		Upper
46D		Lower
46E		Upper
46F		Lower
470		Upper
471		Lower
472		Upper
473		Lower
474		Upper
475		Lower
476		Upper
477		Lower
478		Upper
479		Lower
47A		Upper
47B		Lower
47C		Upper
47D		Lower
47E		Upper
47F		Lower
480		Upper
481		Lower
483	489	Extend
48A		Upper
48B		Lower
48C		Upper
48D		Lower
48E		Upper
48F		Lower
490		Upper
491		Lower
492		Upper
493		Lower
494		Upper
495		Lower
496		Upper
497		Lower
498		Upper
499		Lower
49A		Upper
49B		Lower
49C		Upper
49D		Lower
49E		Upper
49F		Lower
4A0		Upper
4A1		Lower
4A2		Upper
4A3		Lower
4A4		Upper
4A5		Lower
4A6		Upper
4A7		Lower
4A8		Upper
4A9		Lower
4AA		Upper
4AB		Lower
4AC		Upper
4AD		Lower
4AE		Upper
4AF		Lower
4B0		Upper
4B1		Lower
4B2		Upper
4B3		Lower
4B4		Upper
4B5		Lower
4B6		Upper
4B7		Lower
4B8		Upper
4B9		Lower
4BA		Upper
4BB		Lower
4BC		Upper
4BD		Lower
4BE		Upper
4BF		Lower
4C0	4C1	Upper
4C2		Lower
4C3		Upper
4C4		Lower
4C5		Upper
4C6		Lower
4C7		Upper
4C8		Lower
4C9		Upper
4CA		Lower
4CB		Upper
4CC		Lower
4CD		Upper
4CE	4CF	Lower
4D0		Upper
4D1		Lower
4D2		Upper
4D3		Lower
4D4		Upper
4D5		Lower
4D6		Upper
4D7		Lower
4D8		Upper
4D9		Lower
4DA		Upper
4DB		Lower
4DC		Upper
4DD		Lower
4DE		Upper
4DF		Lower
4E0		Upper
4E1		Lower
4E2		Upper
4E3		Lower
4E4		Upper
4E5		Lower
4E6		Upper
4E7		Lower
4E8		Upper
4E9		Lower
4EA		Upper
4EB		Lower
4EC		Upper
4ED		Lower
4EE		Upper
4EF		Lower
4F0		Upper
4F1		Lower
4F2		Upper
4F3		Lower
4F4		Upper
4F5		Lower
4F6		Upper
4F7		Lower
4F8		Upper
4F9		Lower
4FA		Upper
4FB		Lower
4FC		Upper
4FD		Lower
4FE		Upper
4FF		Lower
500		Upper
501		Lower
502		Upper
503		Lower
504		Upper
505		Lower
506		Upper
507		Lower
508		Upper
509		Lower
50A		Upper
50B		Lower
50C		Upper
50D		Lower
50E		Upper
50F		Lower
510		Upper
511		Lower
512		Upper
513		Lower
514		Upper
515		Lower
516		Upper
517		Lower
518		Upper
519		Lower
51A		Upper
51B		Lower
51C		Upper
51D		Lower
51E		Upper
51F		Lower
520		Upper
521		Lower
522		Upper
523		Lower
524		Upper
525		Lower
526		Upper
527		Lower
528		Upper
529		Lower
52A		Upper
52B		Lower
52C		Upper
52D		Lower
52E		Upper
52F		Lower
531	556	Upper
559		OLetter
55D		SContinue
560	588	Lower
589		STerm
591	5BD	Extend
5BF		Extend
5C1	5C2	Extend
5C4	5C5	Extend
5C7		Extend
5D0	5EA	OLetter
5EF	5F3	OLetter
600	605	Format
60C	60D	SContinue
610	61A	Extend
61C		Format
61E	61F	STerm
620	64A	OLetter
64B	65F	Extend
660	669	Numeric
66B	66C	Numeric
66E	66F	OLetter
670		Extend
671	6D3	OLetter
6D4		STerm
6D5		OLetter
6D6	6DC	Extend
6DD		Format
6DF	6E4	Extend
6E5	6E6	OLetter
6E7	6E8	Extend
6EA	6ED	Extend
6EE	6EF	OLetter
6F0	6F9	Numeric
6FA	6FC	OLetter
6FF		OLetter
700	702	STerm
70F		Format
710		OLetter
711		Extend
712	72F	OLetter
730	74A	Extend
74D	7A5	OLetter
7A6	7B0	Extend
7B1		OLetter
7C0	7C9	Numeric
7CA	7EA	OLetter
7EB	7F3	Extend
7F4	7F5	OLetter
7F8		SContinue
7F9		STerm
7FA		OLetter
7FD		Extend
800	815	OLetter
816	819	Extend
81A		OLetter
81B	823	Extend
824		OLetter
825	827	Extend
828		OLetter
829	82D	Extend
837		STerm
839		STerm
83D	83E	STerm
840	858	OLetter
859	85B	Extend
860	86A	OLetter
8A0	8B4	OLetter
8B6	8C7	OLetter
8D3	8E1	Extend
8E2		Format
8E3	903	Extend
904	939	OLetter
93A	93C	Extend
93D		OLetter
93E	94F	Extend
950		OLetter
951	957	Extend
958	961	OLetter
962	963	Extend
964	965	STerm
966	96F	Numeric
971	980	OLetter
981	983	Extend
985	98C	OLetter
98F	990	OLetter
993	9A8	OLetter
9AA	9B0	OLetter
9B2		OLetter
9B6	9B9	OLetter
9BC		Extend
9BD		OLetter
9BE	9C4	Extend
9C7	9C8	Extend
9CB	9CD	Extend
9CE		OLetter
9D7		Extend
9DC	9DD	OLetter
9DF	9E1	OLetter
9E2	9E3	Extend
9E6	9EF	Numeric
9F0	9F1	OLetter
9FC		OLetter
9FE		Extend
A01	A03	Extend
A05	A0A	OLetter
A0F	A10	OLetter
A13	A28	OLetter
A2A	A30	OLetter
A32	A33	OLetter
A35	A36	OLetter
A38	A39	OLetter
A3C		Extend
A3E	A42	Extend
A47	A48	Extend
A4B	A4D	Extend
A51		Extend
A59	A5C	OLetter
A5E		OLetter
A66	A6F	Numeric
A70	A71	Extend
A72	A74	OLetter
A75		Extend
A81	A83	Extend
A85	A8D	OLetter
A8F	A91	OLetter
A93	AA8	OLetter
AAA	AB0	OLetter
AB2	AB3	OLetter
AB5	AB9	OLetter
ABC		Extend
ABD		OLetter
ABE	AC5	Extend
AC7	AC9	Extend
ACB	ACD	Extend
AD0		OLetter
AE0	AE1	OLetter
AE2	AE3	Extend
AE6	AEF	Numeric
AF9		OLetter
AFA	AFF	Extend
B01	B03	Extend
B05	B0C	OLetter
B0F	B10	OLetter
B13	B28	OLetter
B2A	B30	OLetter
B32	B33	OLetter
B35	B39	OLetter
B3C		Extend
B3D		OLetter
B3E	B44	Extend
B47	B48	Extend
B4B	B4D	Extend
B55	B57	Extend
B5C	B5D	OLetter
B5F	B61	OLetter
B62	B63	Extend
B66	B6F	Numeric
B71		OLetter
B82		Extend
B83		OLetter
B85	B8A	OLetter
B8E	B90	OLetter
B92	B95	OLetter
B99	B9A	OLetter
B9C		OLetter
B9E	B9F	OLetter
BA3	BA4	OLetter
BA8	BAA	OLetter
BAE	BB9	OLetter
BBE	BC2	Extend
BC6	BC8	Extend
BCA	BCD	Extend
BD0		OLetter
BD7		Extend
BE6	BEF	Numeric
C00	C04	Extend
C05	C0C	OLetter
C0E	C10	OLetter
C12	C28	OLetter
C2A	C39	OLetter
C3D		OLetter
C3E	C44	Extend
C46	C48	Extend
C4A	C4D	Extend
C55	C56	Extend
C58	C5A	OLetter
C60	C61	OLetter
C62	C63	Extend
C66	C6F	Numeric
C80		OLetter
C81	C83	Extend
C85	C8C	OLetter
C8E	C90	OLetter
C92	CA8	OLetter
CAA	CB3	OLetter
CB5	CB9	OLetter
CBC		Extend
CBD		OLetter
CBE	CC4	Extend
CC6	CC8	Extend
CCA	CCD	Extend
CD5	CD6	Extend
CDE		OLetter
CE0	CE1	OLetter
CE2	CE3	Extend
CE6	CEF	Numeric
CF1	CF2	OLetter
D00	D03	Extend
D04	D0C	OLetter
D0E	D10	OLetter
D12	D3A	OLetter
D3B	D3C	Extend
D3D		OLetter
D3E	D44	Extend
D46	D48	Extend
D4A	D4D	Extend
D4E		OLetter
D54	D56	OLetter
D57		Extend
D5F	D61	OLetter
D62	D63	Extend
D66	D6F	Numeric
D7A	D7F	OLetter
D81	D83	Extend
D85	D96	OLetter
D9A	DB1	OLetter
DB3	DBB	OLetter
DBD		OLetter
DC0	DC6	OLetter
DCA		Extend
DCF	DD4	Extend
DD6		Extend
DD8	DDF	Extend
DE6	DEF	Numeric
DF2	DF3	Extend
E01	E30	OLetter
E31		Extend
E32	E33	OLetter
E34	E3A	Extend
E40	E46	OLetter
E47	E4E	Extend
E50	E59	Numeric
E81	E82	OLetter
E84		OLetter
E86	E8A	OLetter
E8C	EA3	OLetter
EA5		OLetter
EA7	EB0	OLetter
EB1		Extend
EB2	EB3	OLetter
EB4	EBC	Extend
EBD		OLetter
EC0	EC4	OLetter
EC6		OLetter
EC8	ECD	Extend
ED0	ED9	Numeric
EDC	EDF	OLetter
F00		OLetter
F18	F19	Extend
F20	F29	Numeric
F35		Extend
F37		Extend
F39		Extend
F3A	F3D	Close
F3E	F3F	Extend
F40	F47	OLetter
F49	F6C	OLetter
F71	F84	Extend
F86	F87	Extend
F88	F8C	OLetter
F8D	F97	Extend
F99	FBC	Extend
FC6		Extend
1000	102A	OLetter
102B	103E	Extend
103F		OLetter
1040	1049	Numeric
104A	104B	STerm
1050	1055	OLetter
1056	1059	Extend
105A	105D	OLetter
105E	1060	Extend
1061		OLetter
1062	1064	Extend
1065	1066	OLetter
1067	106D	Extend
106E	1070	OLetter
1071	1074	Extend
1075	1081	OLetter
1082	108D	Extend
108E		OLetter
108F		Extend
1090	1099	Numeric
109A	109D	Extend
10A0	10C5	Upper
10C7		Upper
10CD		Upper
10D0	10FA	OLetter
10FC	1248	OLetter
124A	124D	OLetter
1250	1256	OLetter
1258		OLetter
125A	125D	OLetter
1260	1288	OLetter
128A	128D	OLetter
1290	12B0	OLetter
12B2	12B5	OLetter
12B8	12BE	OLetter
12C0		OLetter
12C2	12C5	OLetter
12C8	12D6	OLetter
12D8	1310	OLetter
1312	1315	OLetter
1318	135A	OLetter
135D	135F	Extend
1362		STerm
1367	1368	STerm
1380	138F	OLetter
13A0	13F5	Upper
13F8	13FD	Lower
1401	166C	OLetter
166E		STerm
166F	167F	OLetter
1680		Sp
1681	169A	OLetter
169B	169C	Close
16A0	16EA	OLetter
16EE	16F8	OLetter
1700	170C	OLetter
170E	1711	OLetter
1712	1714	Extend
1720	1731	OLetter
1732	1734	Extend
1735	1736	STerm
1740	1751	OLetter
1752	1753	Extend
1760	176C	OLetter
176E	1770	OLetter
1772	1773	Extend
1780	17B3	OLetter
17B4	17D3	Extend
17D7		OLetter
17DC		OLetter
17DD		Extend
17E0	17E9	Numeric
1802		SContinue
1803		STerm
1808		SContinue
1809		STerm
180B	180D	Extend
180E		Format
1810	1819	Numeric
1820	1878	OLetter
1880	1884	OLetter
1885	1886	Extend
1887	18A8	OLetter
18A9		Extend
18AA		OLetter
18B0	18F5	OLetter
1900	191E	OLetter
1920	192B	Extend
1930	193B	Extend
1944	1945	STerm
1946	194F	Numeric
1950	196D	OLetter
1970	1974	OLetter
1980	19AB	OLetter
19B0	19C9	OLetter
19D0	19D9	Numeric
1A00	1A16	OLetter
1A17	1A1B	Extend
1A20	1A54	OLetter
1A55	1A5E	Extend
1A60	1A7C	Extend
1A7F		Extend
1A80	1A89	Numeric
1A90	1A99	Numeric
1AA7		OLetter
1AA8	1AAB	STerm
1AB0	1AC0	Extend
1B00	1B04	Extend
1B05	1B33	OLetter
1B34	1B44	Extend
1B45	1B4B	OLetter
1B50	1B59	Numeric
1B5A	1B5B	STerm
1B5E	1B5F	STerm
1B6B	1B73	Extend
1B80	1B82	Extend
1B83	1BA0	OLetter
1BA1	1BAD	Extend
1BAE	1BAF	OLetter
1BB0	1BB9	Numeric
1BBA	1BE5	OLetter
1BE6	1BF3	Extend
1C00	1C23	OLetter
1C24	1C37	Extend
1C3B	1C3C	STerm
1C40	1C49	Numeric
1C4D	1C4F	OLetter
1C50	1C59	Numeric
1C5A	1C7D	OLetter
1C7E	1C7F	STerm
1C80	1C88	Lower
1C90	1CBA	OLetter
1CBD	1CBF	OLetter
1CD0	1CD2	Extend
1CD4	1CE8	Extend
1CE9	1CEC	OLetter
1CED		Extend
1CEE	1CF3	OLetter
1CF4		Extend
1CF5	1CF6	OLetter
1CF7	1CF9	Extend
1CFA		OLetter
1D00	1DBF	Lower
1DC0	1DF9	Extend
1DFB	1DFF	Extend
1E00		Upper
1E01		Lower
1E02		Upper
1E03		Lower
1E04		Upper
1E05		Lower
1E06		Upper
1E07		Lower
1E08		Upper
1E09		Lower
1E0A		Upper
1E0B		Lower
1E0C		Upper
1E0D		Lower
1E0E		Upper
1E0F		Lower
1E10		Upper
1E11		Lower
1E12		Upper
1E13		Lower
1E14		Upper
1E15		Lower
1E16		Upper
1E17		Lower
1E18		Upper
1E19		Lower
1E1A		Upper
1E1B		Lower
1E1C		Upper
1E1D		Lower
1E1E		Upper
1E1F		Lower
1E20		Upper
1E21		Lower
1E22		Upper
1E23		Lower
1E24		Upper
1E25		Lower
1E26		Upper
1E27		Lower
1E28		Upper
1E29		Lower
1E2A		Upper
1E2B		Lower
1E2C		Upper
1E2D		Lower
1E2E		Upper
1E2F		Lower
1E30		Upper
1E31		Lower
1E32		Upper
1E33		Lower
1E34		Upper
1E35		Lower
1E36		Upper
1E37		Lower
1E38		Upper
1E39		Lower
1E3A		Upper
1E3B		Lower
1E3C		Upper
1E3D		Lower
1E3E		Upper
1E3F		Lower
1E40		Upper
1E41		Lower
1E42		Upper
1E43		Lower
1E44		Upper
1E45		Lower
1E46		Upper
1E47		Lower
1E48		Upper
1E49		Lower
1E4A		Upper
1E4B		Lower
1E4C		Upper
1E4D		Lower
1E4E		Upper
1E4F		Lower
1E50		Upper
1E51		Lower
1E52		Upper
1E53		Lower
1E54		Upper
1E55		Lower
1E56		Upper
1E57		Lower
1E58		Upper
1E59		Lower
1E5A		Upper
1E5B		Lower
1E5C		Upper
1E5D		Lower
1E5E		Upper
1E5F		Lower
1E60		Upper
1E61		Lower
1E62		Upper
1E63		Lower
1E64		Upper
1E65		Lower
1E66		Upper
1E67		Lower
1E68		Upper
1E69		Lower
1E6A		Upper
1E6B		Lower
1E6C		Upper
1E6D		Lower
1E6E		Upper
1E6F		Lower
1E70		Upper
1E71		Lower
1E72		Upper
1E73		Lower
1E74		Upper
1E75		Lower
1E76		Upper
1E77		Lower
1E78		Upper
1E79		Lower
1E7A		Upper
1E7B		Lower
1E7C		Upper
1E7D		Lower
1E7E		Upper
1E7F		Lower
1E80		Upper
1E81		Lower
1E82		Upper
1E83		Lower
1E84		Upper
1E85		Lower
1E86		Upper
1E87		Lower
1E88		Upper
1E89		Lower
1E8A		Upper
1E8B		Lower
1E8C		Upper
1E8D		Lower
1E8E		Upper
1E8F		Lower
1E90		Upper
1E91		Lower
1E92		Upper
1E93		Lower
1E94		Upper
1E95	1E9D	Lower
1E9E		Upper
1E9F		Lower
1EA0		Upper
1EA1		Lower
1EA2		Upper
1EA3		Lower
1EA4		Upper
1EA5		Lower
1EA6		Upper
1EA7		Lower
1EA8		Upper
1EA9		Lower
1EAA		Upper
1EAB		Lower
1EAC		Upper
1EAD		Lower
1EAE		Upper
1EAF		Lower
1EB0		Upper
1EB1		Lower
1EB2		Upper
1EB3		Lower
1EB4		Upper
1EB5		Lower
1EB6		Upper
1EB7		Lower
1EB8		Upper
1EB9		Lower
1EBA		Upper
1EBB		Lower
1EBC		Upper
1EBD		Lower
1EBE		Upper
1EBF		Lower
1EC0		Upper
1EC1		Lower
1EC2		Upper
1EC3		Lower
1EC4		Upper
1EC5		Lower
1EC6		Upper
1EC7		Lower
1EC8		Upper
1EC9		Lower
1ECA		Upper
1ECB		Lower
1ECC		Upper
1ECD		Lower
1ECE		Upper
1ECF		Lower
1ED0		Upper
1ED1		Lower
1ED2		Upper
1ED3		Lower
1ED4		Upper
1ED5		Lower
1ED6		Upper
1ED7		Lower
1ED8		Upper
1ED9		Lower
1EDA		Upper
1EDB		Lower
1EDC		Upper
1EDD		Lower
1EDE		Upper
1EDF		Lower
1EE0		Upper
1EE1		Lower
1EE2		Upper
1EE3		Lower
1EE4		Upper
1EE5		Lower
1EE6		Upper
1EE7		Lower
1EE8		Upper
1EE9		Lower
1EEA		Upper
1EEB		Lower
1EEC		Upper
1EED		Lower
1EEE		Upper
1EEF		Lower
1EF0		Upper
1EF1		Lower
1EF2		Upper
1EF3		Lower
1EF4		Upper
1EF5		Lower
1EF6		Upper
1EF7		Lower
1EF8		Upper
1EF9		Lower
1EFA		Upper
1EFB		Lower
1EFC		Upper
1EFD		Lower
1EFE		Upper
1EFF	1F07	Lower
1F08	1F0F	Upper
1F10	1F15	Lower
1F18	1F1D	Upper
1F20	1F27	Lower
1F28	1F2F	Upper
1F30	1F37	Lower
1F38	1F3F	Upper
1F40	1F45	Lower
1F48	1F4D	Upper
1F50	1F57	Lower
1F59		Upper
1F5B		Upper
1F5D		Upper
1F5F		Upper
1F60	1F67	Lower
1F68	1F6F	Upper
1F70	1F7D	Lower
1F80	1F87	Lower
1F88	1F8F	Upper
1F90	1F97	Lower
1F98	1F9F	Upper
1FA0	1FA7	Lower
1FA8	1FAF	Upper
1FB0	1FB4	Lower
1FB6	1FB7	Lower
1FB8	1FBC	Upper
1FBE		Lower
1FC2	1FC4	Lower
1FC6	1FC7	Lower
1FC8	1FCC	Upper
1FD0	1FD3	Lower
1FD6	1FD7	Lower
1FD8	1FDB	Upper
1FE0	1FE7	Lower
1FE8	1FEC	Upper
1FF2	1FF4	Lower
1FF6	1FF7	Lower
1FF8	1FFC	Upper
2000	200A	Sp
200B		Format
200C	200D	Extend
200E	200F	Format
2013	2014	SContinue
2018	201F	Close
2024		ATerm
2028	2029	Sep
202A	202E	Format
202F		Sp
2039	203A	Close
203C	203D	STerm
2045	2046	Close
2047	2049	STerm
205F		Sp
2060	2064	Format
2066	206F	Format
2071		Lower
207D	207E	Close
207F		Lower
208D	208E	Close
2090	209C	Lower
20D0	20F0	Extend
2102		Upper
2107		Upper
210A		Lower
210B	210D	Upper
210E	210F	Lower
2110	2112	Upper
2113		Lower
2115		Upper
2119	211D	Upper
2124		Upper
2126		Upper
2128		Upper
212A	212D	Upper
212F		Lower
2130	2133	Upper
2134		Lower
2135	2138	OLetter
2139		Lower
213C	213D	Lower
213E	213F	Upper
2145		Upper
2146	2149	Lower
214E		Lower
2160	216F	Upper
2170	217F	Lower
2180	2182	OLetter
2183		Upper
2184		Lower
2185	2188	OLetter
2308	230B	Close
2329	232A	Close
24B6	24CF	Upper
24D0	24E9	Lower
275B	2760	Close
2768	2775	Close
27C5	27C6	Close
27E6	27EF	Close
2983	2998	Close
29D8	29DB	Close
29FC	29FD	Close
2C00	2C2E	Upper
2C30	2C5E	Lower
2C60		Upper
2C61		Lower
2C62	2C64	Upper
2C65	2C66	Lower
2C67		Upper
2C68		Lower
2C69		Upper
2C6A		Lower
2C6B		Upper
2C6C		Lower
2C6D	2C70	Upper
2C71		Lower
2C72		Upper
2C73	2C74	Lower
2C75		Upper
2C76	2C7D	Lower
2C7E	2C80	Upper
2C81		Lower
2C82		Upper
2C83		Lower
2C84		Upper
2C85		Lower
2C86		Upper
2C87		Lower
2C88		Upper
2C89		Lower
2C8A		Upper
2C8B		Lower
2C8C		Upper
2C8D		Lower
2C8E		Upper
2C8F		Lower
2C90		Upper
2C91		Lower
2C92		Upper
2C93		Lower
2C94		Upper
2C95		Lower
2C96		Upper
2C97		Lower
2C98		Upper
2C99		Lower
2C9A		Upper
2C9B		Lower
2C9C		Upper
2C9D		Lower
2C9E		Upper
2C9F		Lower
2CA0		Upper
2CA1		Lower
2CA2		Upper
2CA3		Lower
2CA4		Upper
2CA5		Lower
2CA6		Upper
2CA7		Lower
2CA8		Upper
2CA9		Lower
2CAA		Upper
2CAB		Lower
2CAC		Upper
2CAD		Lower
2CAE		Upper
2CAF		Lower
2CB0		Upper
2CB1		Lower
2CB2		Upper
2CB3		Lower
2CB4		Upper
2CB5		Lower
2CB6		Upper
2CB7		Lower
2CB8		Upper
2CB9		Lower
2CBA		Upper
2CBB		Lower
2CBC		Upper
2CBD		Lower
2CBE		Upper
2CBF		Lower
2CC0		Upper
2CC1		Lower
2CC2		Upper
2CC3		Lower
2CC4		Upper
2CC5		Lower
2CC6		Upper
2CC7		Lower
2CC8		Upper
2CC9		Lower
2CCA		Upper
2CCB		Lower
2CCC		Upper
2CCD		Lower
2CCE		Upper
2CCF		Lower
2CD0		Upper
2CD1		Lower
2CD2		Upper
2CD3		Lower
2CD4		Upper
2CD5		Lower
2CD6		Upper
2CD7		Lower
2CD8		Upper
2CD9		Lower
2CDA		Upper
2CDB		Lower
2CDC		Upper
2CDD		Lower
2CDE		Upper
2CDF		Lower
2CE0		Upper
2CE1		Lower
2CE2		Upper
2CE3	2CE4	Lower
2CEB		Upper
2CEC		Lower
2CED		Upper
2CEE		Lower
2CEF	2CF1	Extend
2CF2		Upper
2CF3		Lower
2D00	2D25	Lower
2D27		Lower
2D2D		Lower
2D30	2D67	OLetter
2D6F		OLetter
2D7F		Extend
2D80	2D96	OLetter
2DA0	2DA6	OLetter
2DA8	2DAE	OLetter
2DB0	2DB6	OLetter
2DB8	2DBE	OLetter
2DC0	2DC6	OLetter
2DC8	2DCE	OLetter
2DD0	2DD6	OLetter
2DD8	2DDE	OLetter
2DE0	2DFF	Extend
2E00	2E0D	Close
2E1C	2E1D	Close
2E20	2E29	Close
2E2E		STerm
2E2F		OLetter
2E3C		STerm
2E42		Close
3000		Sp
3001		SContinue
3002		STerm
3005	3007	OLetter
3008	3011	Close
3014	301B	Close
301D	301F	Close
3021	3029	OLetter
302A	302F	Extend
3031	3035	OLetter
3038	303C	OLetter
3041	3096	OLetter
3099	309A	Extend
309D	309F	OLetter
30A1	30FA	OLetter
30FC	30FF	OLetter
3105	312F	OLetter
3131	318E	OLetter
31A0	31BF	OLetter
31F0	31FF	OLetter
3400	4DBF	OLetter
4E00	9FFC	OLetter
A000	A48C	OLetter
A4D0	A4FD	OLetter
A4FF		STerm
A500	A60C	OLetter
A60E	A60F	STerm
A610	A61F	OLetter
A620	A629	Numeric
A62A	A62B	OLetter
A640		Upper
A641		Lower
A642		Upper
A643		Lower
A644		Upper
A645		Lower
A646		Upper
A647		Lower
A648		Upper
A649		Lower
A64A		Upper
A64B		Lower
A64C		Upper
A64D		Lower
A64E		Upper
A64F		Lower
A650		Upper
A651		Lower
A652		Upper
A653		Lower
A654		Upper
A655		Lower
A656		Upper
A657		Lower
A658		Upper
A659		Lower
A65A		Upper
A65B		Lower
A65C		Upper
A65D		Lower
A65E		Upper
A65F		Lower
A660		Upper
A661		Lower
A662		Upper
A663		Lower
A664		Upper
A665		Lower
A666		Upper
A667		Lower
A668		Upper
A669		Lower
A66A		Upper
A66B		Lower
A66C		Upper
A66D		Lower
A66E		OLetter
A66F	A672	Extend
A674	A67D	Extend
A67F		OLetter
A680		Upper
A681		Lower
A682		Upper
A683		Lower
A684		Upper
A685		Lower
A686		Upper
A687		Lower
A688		Upper
A689		Lower
A68A		Upper
A68B		Lower
A68C		Upper
A68D		Lower
A68E		Upper
A68F		Lower
A690		Upper
A691		Lower
A692		Upper
A693		Lower
A694		Upper
A695		Lower
A696		Upper
A697		Lower
A698		Upper
A699		Lower
A69A		Upper
A69B	A69D	Lower
A69E	A69F	Extend
A6A0	A6EF	OLetter
A6F0	A6F1	Extend
A6F3		STerm
A6F7		STerm
A717	A71F	OLetter
A722		Upper
A723		Lower
A724		Upper
A725		Lower
A726		Upper
A727		Lower
A728		Upper
A729		Lower
A72A		Upper
A72B		Lower
A72C		Upper
A72D		Lower
A72E		Upper
A72F	A731	Lower
A732		Upper
A733		Lower
A734		Upper
A735		Lower
A736		Upper
A737		Lower
A738		Upper
A739		Lower
A73A		Upper
A73B		Lower
A73C		Upper
A73D		Lower
A73E		Upper
A73F		Lower
A740		Upper
A741		Lower
A742		Upper
A743		Lower
A744		Upper
A745		Lower
A746		Upper
A747		Lower
A748		Upper
A749		Lower
A74A		Upper
A74B		Lower
A74C		Upper
A74D		Lower
A74E		Upper
A74F		Lower
A750		Upper
A751		Lower
A752		Upper
A753		Lower
A754		Upper
A755		Lower
A756		Upper
A757		Lower
A758		Upper
A759		Lower
A75A		Upper
A75B		Lower
A75C		Upper
A75D		Lower
A75E		Upper
A75F		Lower
A760		Upper
A761		Lower
A762		Upper
A763		Lower
A764		Upper
A765		Lower
A766		Upper
A767		Lower
A768		Upper
A769		Lower
A76A		Upper
A76B		Lower
A76C		Upper
A76D		Lower
A76E		Upper
A76F	A778	Lower
A779		Upper
A77A		Lower
A77B		Upper
A77C		Lower
A77D	A77E	Upper
A77F		Lower
A780		Upper
A781		Lower
A782		Upper
A783		Lower
A784		Upper
A785		Lower
A786		Upper
A787		Lower
A788		OLetter
A78B		Upper
A78C		Lower
A78D		Upper
A78E		Lower
A78F		OLetter
A790		Upper
A791		Lower
A792		Upper
A793	A795	Lower
A796		Upper
A797		Lower
A798		Upper
A799		Lower
A79A		Upper
A79B		Lower
A79C		Upper
A79D		Lower
A79E		Upper
A79F		Lower
A7A0		Upper
A7A1		Lower
A7A2		Upper
A7A3		Lower
A7A4		Upper
A7A5		Lower
A7A6		Upper
A7A7		Lower
A7A8		Upper
A7A9		Lower
A7AA	A7AE	Upper
A7AF		Lower
A7B0	A7B4	Upper
A7B5		Lower
A7B6		Upper
A7B7		Lower
A7B8		Upper
A7B9		Lower
A7BA		Upper
A7BB		Lower
A7BC		Upper
A7BD		Lower
A7BE		Upper
A7BF		Lower
A7C2		Upper
A7C3		Lower
A7C4	A7C7	Upper
A7C8		Lower
A7C9		Upper
A7CA		Lower
A7F5		Upper
A7F6		Lower
A7F7		OLetter
A7F8	A7FA	Lower
A7FB	A801	OLetter
A802		Extend
A803	A805	OLetter
A806		Extend
A807	A80A	OLetter
A80B		Extend
A80C	A822	OLetter
A823	A827	Extend
A82C		Extend
A840	A873	OLetter
A876	A877	STerm
A880	A881	Extend
A882	A8B3	OLetter
A8B4	A8C5	Extend
A8CE	A8CF	STerm
A8D0	A8D9	Numeric
A8E0	A8F1	Extend
A8F2	A8F7	OLetter
A8FB		OLetter
A8FD	A8FE	OLetter
A8FF		Extend
A900	A909	Numeric
A90A	A925	OLetter
A926	A92D	Extend
A92F		STerm
A930	A946	OLetter
A947	A953	Extend
A960	A97C	OLetter
A980	A983	Extend
A984	A9B2	OLetter
A9B3	A9C0	Extend
A9C8	A9C9	STerm
A9CF		OLetter
A9D0	A9D9	Numeric
A9E0	A9E4	OLetter
A9E5		Extend
A9E6	A9EF	OLetter
A9F0	A9F9	Numeric
A9FA	A9FE	OLetter
AA00	AA28	OLetter
AA29	AA36	Extend
AA40	AA42	OLetter
AA43		Extend
AA44	AA4B	OLetter
AA4C	AA4D	Extend
AA50	AA59	Numeric
AA5D	AA5F	STerm
AA60	AA76	OLetter
AA7A		OLetter
AA7B	AA7D	Extend
AA7E	AAAF	OLetter
AAB0		Extend
AAB1		OLetter
AAB2	AAB4	Extend
AAB5	AAB6	OLetter
AAB7	AAB8	Extend
AAB9	AABD	OLetter
AABE	AABF	Extend
AAC0		OLetter
AAC1		Extend
AAC2		OLetter
AADB	AADD	OLetter
AAE0	AAEA	OLetter
AAEB	AAEF	Extend
AAF0	AAF1	STerm
AAF2	AAF4	OLetter
AAF5	AAF6	Extend
AB01	AB06	OLetter
AB09	AB0E	OLetter
AB11	AB16	OLetter
AB20	AB26	OLetter
AB28	AB2E	OLetter
AB30	AB5A	Lower
AB5C	AB68	Lower
AB69		OLetter
AB70	ABBF	Lower
ABC0	ABE2	OLetter
ABE3	ABEA	Extend
ABEB		STerm
ABEC	ABED	Extend
ABF0	ABF9	Numeric
AC00	D7A3	OLetter
D7B0	D7C6	OLetter
D7CB	D7FB	OLetter
F900	FA6D	OLetter
FA70	FAD9	OLetter
FB00	FB06	Lower
FB13	FB17	Lower
FB1D		OLetter
FB1E		Extend
FB1F	FB28	OLetter
FB2A	FB36	OLetter
FB38	FB3C	OLetter
FB3E		OLetter
FB40	FB41	OLetter
FB43	FB44	OLetter
FB46	FBB1	OLetter
FBD3	FD3D	OLetter
FD3E	FD3F	Close
FD50	FD8F	OLetter
FD92	FDC7	OLetter
FDF0	FDFB	OLetter
FE00	FE0F	Extend
FE10	FE11	SContinue
FE13		SContinue
FE17	FE18	Close
FE20	FE2F	Extend
FE31	FE32	SContinue
FE35	FE44	Close
FE47	FE48	Close
FE50	FE51	SContinue
FE52		ATerm
FE55		SContinue
FE56	FE57	STerm
FE58		SContinue
FE59	FE5E	Close
FE63		SContinue
FE70	FE74	OLetter
FE76	FEFC	OLetter
FEFF		Format
FF01		STerm
FF08	FF09	Close
FF0C	FF0D	SContinue
FF0E		ATerm
FF10	FF19	Numeric
FF1A		SContinue
FF1F		STerm
FF21	FF3A	Upper
FF3B		Close
FF3D		Close
FF41	FF5A	Lower
FF5B		Close
FF5D		Close
FF5F	FF60	Close
FF61		STerm
FF62	FF63	Close
FF64		SContinue
FF66	FF9D	OLetter
FF9E	FF9F	Extend
FFA0	FFBE	OLetter
FFC2	FFC7	OLetter
FFCA	FFCF	OLetter
FFD2	FFD7	OLetter
FFDA	FFDC	OLetter
FFF9	FFFB	Format
10000	1000B	OLetter
1000D	10026	OLetter
10028	1003A	OLetter
1003C	1003D	OLetter
1003F	1004D	OLetter
10050	1005D	OLetter
10080	100FA	OLetter
10140	10174	OLetter
101FD		Extend
10280	1029C	OLetter
102A0	102D0	OLetter
102E0		Extend
10300	1031F	OLetter
1032D	1034A	OLetter
10350	10375	OLetter
10376	1037A	Extend
10380	1039D	OLetter
103A0	103C3	OLetter
103C8	103CF	OLetter
103D1	103D5	OLetter
10400	10427	Upper
10428	1044F	Lower
10450	1049D	OLetter
104A0	104A9	Numeric
104B0	104D3	Upper
104D8	104FB	Lower
10500	10527	OLetter
10530	10563	OLetter
10600	10736	OLetter
10740	10755	OLetter
10760	10767	OLetter
10800	10805	OLetter
10808		OLetter
1080A	10835	OLetter
10837	10838	OLetter
1083C		OLetter
1083F	10855	OLetter
10860	10876	OLetter
10880	1089E	OLetter
108E0	108F2	OLetter
108F4	108F5	OLetter
10900	10915	OLetter
10920	10939	OLetter
10980	109B7	OLetter
109BE	109BF	OLetter
10A00		OLetter
10A01	10A03	Extend
10A05	10A06	Extend
10A0C	10A0F	Extend
10A10	10A13	OLetter
10A15	10A17	OLetter
10A19	10A35	OLetter
10A38	10A3A	Extend
10A3F		Extend
10A56	10A57	STerm
10A60	10A7C	OLetter
10A80	10A9C	OLetter
10AC0	10AC7	OLetter
10AC9	10AE4	OLetter
10AE5	10AE6	Extend
10B00	10B35	OLetter
10B40	10B55	OLetter
10B60	10B72	OLetter
10B80	10B91	OLetter
10C00	10C48	OLetter
10C80	10CB2	Upper
10CC0	10CF2	Lower
10D00	10D23	OLetter
10D24	10D27	Extend
10D30	10D39	Numeric
10E80	10EA9	OLetter
10EAB	10EAC	Extend
10EB0	10EB1	OLetter
10F00	10F1C	OLetter
10F27		OLetter
10F30	10F45	OLetter
10F46	10F50	Extend
10F55	10F59	STerm
10FB0	10FC4	OLetter
10FE0	10FF6	OLetter
11000	11002	Extend
11003	11037	OLetter
11038	11046	Extend
11047	11048	STerm
11066	1106F	Numeric
1107F	11082	Extend
11083	110AF	OLetter
110B0	110BA	Extend
110BD		Format
110BE	110C1	STerm
110CD		Format
110D0	110E8	OLetter
110F0	110F9	Numeric
11100	11102	Extend
11103	11126	OLetter
11127	11134	Extend
11136	1113F	Numeric
11141	11143	STerm
11144		OLetter
11145	11146	Extend
11147		OLetter
11150	11172	OLetter
11173		Extend
11176		OLetter
11180	11182	Extend
11183	111B2	OLetter
111B3	111C0	Extend
111C1	111C4	OLetter
111C5	111C6	STerm
111C9	111CC	Extend
111CD		STerm
111CE	111CF	Extend
111D0	111D9	Numeric
111DA		OLetter
111DC		OLetter
111DE	111DF	STerm
11200	11211	OLetter
11213	1122B	OLetter
1122C	11237	Extend
11238	11239	STerm
1123B	1123C	STerm
1123E		Extend
11280	11286	OLetter
11288		OLetter
1128A	1128D	OLetter
1128F	1129D	OLetter
1129F	112A8	OLetter
112A9		STerm
112B0	112DE	OLetter
112DF	112EA	Extend
112F0	112F9	Numeric
11300	11303	Extend
11305	1130C	OLetter
1130F	11310	OLetter
11313	11328	OLetter
1132A	11330	OLetter
11332	11333	OLetter
11335	11339	OLetter
1133B	1133C	Extend
1133D		OLetter
1133E	11344	Extend
11347	11348	Extend
1134B	1134D	Extend
11350		OLetter
11357		Extend
1135D	11361	OLetter
11362	11363	Extend
11366	1136C	Extend
11370	11374	Extend
11400	11434	OLetter
11435	11446	Extend
11447	1144A	OLetter
1144B	1144C	STerm
11450	11459	Numeric
1145E		Extend
1145F	11461	OLetter
11480	114AF	OLetter
114B0	114C3	Extend
114C4	114C5	OLetter
114C7		OLetter
114D0	114D9	Numeric
11580	115AE	OLetter
115AF	115B5	Extend
115B8	115C0	Extend
115C2	115C3	STerm
115C9	115D7	STerm
115D8	115DB	OLetter
115DC	115DD	Extend
11600	1162F	OLetter
11630	11640	Extend
11641	11642	STerm
11644		OLetter
11650	11659	Numeric
11680	116AA	OLetter
116AB	116B7	Extend
116B8		OLetter
116C0	116C9	Numeric
11700	1171A	OLetter
1171D	1172B	Extend
11730	11739	Numeric
1173C	1173E	STerm
11800	1182B	OLetter
1182C	1183A	Extend
118A0	118BF	Upper
118C0	118DF	Lower
118E0	118E9	Numeric
118FF	11906	OLetter
11909		OLetter
1190C	11913	OLetter
11915	11916	OLetter
11918	1192F	OLetter
11930	11935	Extend
11937	11938	Extend
1193B	1193E	Extend
1193F		OLetter
11940		Extend
11941		OLetter
11942	11943	Extend
11944		STerm
11946		STerm
11950	11959	Numeric
119A0	119A7	OLetter
119AA	119D0	OLetter
119D1	119D7	Extend
119DA	119E0	Extend
119E1		OLetter
119E3		OLetter
119E4		Extend
11A00		OLetter
11A01	11A0A	Extend
11A0B	11A32	OLetter
11A33	11A39	Extend
11A3A		OLetter
11A3B	11A3E	Extend
11A42	11A43	STerm
11A47		Extend
11A50		OLetter
11A51	11A5B	Extend
11A5C	11A89	OLetter
11A8A	11A99	Extend
11A9B	11A9C	STerm
11A9D		OLetter
11AC0	11AF8	OLetter
11C00	11C08	OLetter
11C0A	11C2E	OLetter
11C2F	11C36	Extend
11C38	11C3F	Extend
11C40		OLetter
11C41	11C42	STerm
11C50	11C59	Numeric
11C72	11C8F	OLetter
11C92	11CA7	Extend
11CA9	11CB6	Extend
11D00	11D06	OLetter
11D08	11D09	OLetter
11D0B	11D30	OLetter
11D31	11D36	Extend
11D3A		Extend
11D3C	11D3D	Extend
11D3F	11D45	Extend
11D46		OLetter
11D47		Extend
11D50	11D59	Numeric
11D60	11D65	OLetter
11D67	11D68	OLetter
11D6A	11D89	OLetter
11D8A	11D8E	Extend
11D90	11D91	Extend
11D93	11D97	Extend
11D98		OLetter
11DA0	11DA9	Numeric
11EE0	11EF2	OLetter
11EF3	11EF6	Extend
11EF7	11EF8	STerm
11FB0		OLetter
12000	12399	OLetter
12400	1246E	OLetter
12480	12543	OLetter
13000	1342E	OLetter
13430	13438	Format
14400	14646	OLetter
16800	16A38	OLetter
16A40	16A5E	OLetter
16A60	16A69	Numeric
16A6E	16A6F	STerm
16AD0	16AED	OLetter
16AF0	16AF4	Extend
16AF5		STerm
16B00	16B2F	OLetter
16B30	16B36	Extend
16B37	16B38	STerm
16B40	16B43	OLetter
16B44		STerm
16B50	16B59	Numeric
16B63	16B77	OLetter
16B7D	16B8F	OLetter
16E40	16E5F	Upper
16E60	16E7F	Lower
16E98		STerm
16F00	16F4A	OLetter
16F4F		Extend
16F50		OLetter
16F51	16F87	Extend
16F8F	16F92	Extend
16F93	16F9F	OLetter
16FE0	16FE1	OLetter
16FE3		OLetter
16FE4		Extend
16FF0	16FF1	Extend
17000	187F7	OLetter
18800	18CD5	OLetter
18D00	18D08	OLetter
1B000	1B11E	OLetter
1B150	1B152	OLetter
1B164	1B167	OLetter
1B170	1B2FB	OLetter
1BC00	1BC6A	OLetter
1BC70	1BC7C	OLetter
1BC80	1BC88	OLetter
1BC90	1BC99	OLetter
1BC9D	1BC9E	Extend
1BC9F		STerm
1BCA0	1BCA3	Format
1D165	1D169	Extend
1D16D	1D172	Extend
1D173	1D17A	Format
1D17B	1D182	Extend
1D185	1D18B	Extend
1D1AA	1D1AD	Extend
1D242	1D244	Extend
1D400	1D419	Upper
1D41A	1D433	Lower
1D434	1D44D	Upper
1D44E	1D454	Lower
1D456	1D467	Lower
1D468	1D481	Upper
1D482	1D49B	Lower
1D49C		Upper
1D49E	1D49F	Upper
1D4A2		Upper
1D4A5	1D4A6	Upper
1D4A9	1D4AC	Upper
1D4AE	1D4B5	Upper
1D4B6	1D4B9	Lower
1D4BB		Lower
1D4BD	1D4C3	Lower
1D4C5	1D4CF	Lower
1D4D0	1D4E9	Upper
1D4EA	1D503	Lower
1D504	1D505	Upper
1D507	1D50A	Upper
1D50D	1D514	Upper
1D516	1D51C	Upper
1D51E	1D537	Lower
1D538	1D539	Upper
1D53B	1D53E	Upper
1D540	1D544	Upper
1D546		Upper
1D54A	1D550	Upper
1D552	1D56B	Lower
1D56C	1D585	Upper
1D586	1D59F	Lower
1D5A0	1D5B9	Upper
1D5BA	1D5D3	Lower
1D5D4	1D5ED	Upper
1D5EE	1D607	Lower
1D608	1D621	Upper
1D622	1D63B	Lower
1D63C	1D655	Upper
1D656	1D66F	Lower
1D670	1D689	Upper
1D68A	1D6A5	Lower
1D6A8	1D6C0	Upper
1D6C2	1D6DA	Lower
1D6DC	1D6E1	Lower
1D6E2	1D6FA	Upper
1D6FC	1D714	Lower
1D716	1D71B	Lower
1D71C	1D734	Upper
1D736	1D74E	Lower
1D750	1D755	Lower
1D756	1D76E	Upper
1D770	1D788	Lower
1D78A	1D78F	Lower
1D790	1D7A8	Upper
1D7AA	1D7C2	Lower
1D7C4	1D7C9	Lower
1D7CA		Upper
1D7CB		Lower
1D7CE	1D7FF	Numeric
1DA00	1DA36	Extend
1DA3B	1DA6C	Extend
1DA75		Extend
1DA84		Extend
1DA88		STerm
1DA9B	1DA9F	Extend
1DAA1	1DAAF	Extend
1E000	1E006	Extend
1E008	1E018	Extend
1E01B	1E021	Extend
1E023	1E024	Extend
1E026	1E02A	Extend
1E100	1E12C	OLetter
1E130	1E136	Extend
1E137	1E13D	OLetter
1E140	1E149	Numeric
1E14E		OLetter
1E2C0	1E2EB	OLetter
1E2EC	1E2EF	Extend
1E2F0	1E2F9	Numeric
1E800	1E8C4	OLetter
1E8D0	1E8D6	Extend
1E900	1E921	Upper
1E922	1E943	Lower
1E944	1E94A	Extend
1E94B		OLetter
1E950	1E959	Numeric
1EE00	1EE03	OLetter
1EE05	1EE1F	OLetter
1EE21	1EE22	OLetter
1EE24		OLetter
1EE27		OLetter
1EE29	1EE32	OLetter
1EE34	1EE37	OLetter
1EE39		OLetter
1EE3B		OLetter
1EE42		OLetter
1EE47		OLetter
1EE49		OLetter
1EE4B		OLetter
1EE4D	1EE4F	OLetter
1EE51	1EE52	OLetter
1EE54		OLetter
1EE57		OLetter
1EE59		OLetter
1EE5B		OLetter
1EE5D		OLetter
1EE5F		OLetter
1EE61	1EE62	OLetter
1EE64		OLetter
1EE67	1EE6A	OLetter
1EE6C	1EE72	OLetter
1EE74	1EE77	OLetter
1EE79	1EE7C	OLetter
1EE7E		OLetter
1EE80	1EE89	OLetter
1EE8B	1EE9B	OLetter
1EEA1	1EEA3	OLetter
1EEA5	1EEA9	OLetter
1EEAB	1EEBB	OLetter
1F130	1F149	Upper
1F150	1F169	Upper
1F170	1F189	Upper
1F676	1F678	Close
1FBF0	1FBF9	Numeric
20000	2A6DD	OLetter
2A700	2B734	OLetter
2B740	2B81D	OLetter
2B820	2CEA1	OLetter
2CEB0	2EBE0	OLetter
2F800	2FA1D	OLetter
30000	3134A	OLetter
E0001		Format
E0020	E007F	Extend
E0100	E01EF	Extend
END
