# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 13.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V38
178
180
2558
2559
3893
3894
3895
3896
4038
4039
6091
6092
6094
6097
6099
6100
6109
6110
6459
6460
6779
6781
6783
6784
7222
7223
7675
7676
8308
8309
8322
8325
70089
70090
70750
70751
72243
72244
END
